==> tutti-ui-swing-1.0-sources.jar: fr/ifremer/tutti/ui/swing/TuttiExceptionHandler.java <==
package fr.ifremer.tutti.ui.swing;

/*
 * #%L
 * Tutti :: UI
 * $Id: TuttiExceptionHandler.java 418 2013-02-14 23:59:54Z tchemit $
 * $HeadURL: http://svn.forge.codelutin.com/svn/tutti/tags/tutti-1.0/tutti-ui-swing/src/main/java/fr/ifremer/tutti/ui/swing/TuttiExceptionHandler.java $
 * %%
 * Copyright (C) 2012 - 2013 Ifremer
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-3.0.html>.
 * #L%
 */

import fr.ifremer.tutti.service.TuttiTechnicalException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Tutti global exception handler.
 * <p/>
 * Catch all application uncaught and display it in a custom JoptionPane
 * or JXErrorPane.
 * <p/>
 * See http://stackoverflow.com/a/4448569/1165234 for details.
 *
 * @author tchemit <chemit@codelutin.com>
 * @since 1.0
 */
public class TuttiExceptionHandler implements Thread.UncaughtExceptionHandler {

    private static final Log log =
            LogFactory.getLog(TuttiExceptionHandler.class);

    public static class TuttiActionException extends RuntimeException {

        private static final long serialVersionUID = 1L;

        private final AbstractTuttiAction action;

        public TuttiActionException(AbstractTuttiAction action,
                                    Throwable cause) {
            super(cause);
            this.action = action;
        }

        public AbstractTuttiAction getAction() {
            return action;
        }
    }

    @Override
    public void uncaughtException(Thread t, Throwable ex) {
        handleException(t.getName(), ex);
    }

    public void handle(Throwable thrown) {
        // for EDT exceptions
        handleException(Thread.currentThread().getName(), thrown);
    }

    protected void handleException(String tname, Throwable ex) {
        if (log.isErrorEnabled()) {
            log.error("Global application exception [" + tname + "]", ex);
        }

        Throwable cause = ex;

        if (cause instanceof TuttiTechnicalException) {
            cause = cause.getCause();
        }

        boolean backToScreen = false;

        AbstractTuttiAction action = null;
        if (cause instanceof TuttiActionException) {

            TuttiActionException actionException = (TuttiActionException) cause;
            cause = cause.getCause();

            // close action ui

            action = actionException.getAction();

            if (action instanceof AbstractChangeScreenAction) {
                backToScreen = true;
            }
            action.getContext().getActionUI().getModel().setAction(null);
        }

        TuttiUIContext.getErrorHelper().showErrorDialog(cause.getMessage(), ex);

        if (backToScreen) {
            action.getContext().setScreen(TuttiScreen.SELECT_CRUISE);
        }
    }

}

==> tutti-ui-swing-1.0.1-sources.jar: fr/ifremer/tutti/ui/swing/TuttiExceptionHandler.java <==
package fr.ifremer.tutti.ui.swing;

/*
 * #%L
 * Tutti :: UI
 * $Id: TuttiExceptionHandler.java 475 2013-02-25 19:43:58Z tchemit $
 * $HeadURL: http://svn.forge.codelutin.com/svn/tutti/tags/tutti-1.0.1/tutti-ui-swing/src/main/java/fr/ifremer/tutti/ui/swing/TuttiExceptionHandler.java $
 * %%
 * Copyright (C) 2012 - 2013 Ifremer
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-3.0.html>.
 * #L%
 */

import fr.ifremer.tutti.service.TuttiTechnicalException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Tutti global exception handler.
 * <p/>
 * Catch all application uncaught and display it in a custom JoptionPane
 * or JXErrorPane.
 * <p/>
 * See http://stackoverflow.com/a/4448569/1165234 for details.
 *
 * @author tchemit <chemit@codelutin.com>
 * @since 1.0
 */
public class TuttiExceptionHandler implements Thread.UncaughtExceptionHandler {

    private static final Log log =
            LogFactory.getLog(TuttiExceptionHandler.class);

    public static class TuttiActionException extends RuntimeException {

        private static final long serialVersionUID = 1L;

        private final AbstractTuttiAction action;

        public TuttiActionException(AbstractTuttiAction action,
                                    Throwable cause) {
            super(cause);
            this.action = action;
        }

        public AbstractTuttiAction getAction() {
            return action;
        }
    }

    @Override
    public void uncaughtException(Thread t, Throwable ex) {
        handleException(t.getName(), ex);
    }

    public void handle(Throwable thrown) {
        // for EDT exceptions
        handleException(Thread.currentThread().getName(), thrown);
    }

    protected void handleException(String tname, Throwable ex) {
        if (log.isErrorEnabled()) {
            log.error("Global application exception [" + tname + "]", ex);
        }

        Throwable cause = ex;

        if (cause instanceof NullPointerException) {

            // See http://forge.codelutin.com/issues/2055
            if (ex.getStackTrace().length > 0 &&
                ex.getStackTrace()[0].getClassName().contains("swing.")) {
                if (log.isWarnEnabled()) {
                    log.warn("AWT bad error (See http://forge.codelutin.com/issues/2055)", ex);
                }
                return;
            }
        }

        if (cause instanceof IndexOutOfBoundsException) {
            // See http://forge.codelutin.com/issues/2055
            if (ex.getStackTrace().length > 0 &&
                "convertRowIndexToModel".equals(ex.getStackTrace()[0].getMethodName())) {
                if (log.isWarnEnabled()) {
                    log.warn("AWT bad error (See http://forge.codelutin.com/issues/2055)", ex);
                }
                return;
            }
        }
        if (cause instanceof TuttiTechnicalException) {
            cause = cause.getCause();
        }

        boolean backToScreen = false;

        AbstractTuttiAction action = null;
        if (cause instanceof TuttiActionException) {

            TuttiActionException actionException = (TuttiActionException) cause;
            cause = cause.getCause();

            // close action ui

            action = actionException.getAction();

            if (action instanceof AbstractChangeScreenAction) {
                backToScreen = true;
            }
            action.getContext().getActionUI().getModel().setAction(null);
        }

        TuttiUIContext.getErrorHelper().showErrorDialog(cause.getMessage(), ex);

        if (backToScreen) {
            action.getContext().setScreen(TuttiScreen.SELECT_CRUISE);
        }
    }

}
